/*
 * PROJECT			: Database integration library
 *
 * COPYRIGHT        : Copyright (C) 2000 Tim Stephenson
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: ConnectionFactory.java,v $
 * Revision 1.3  2001/12/03 12:42:31  tstephen
 * changes to get the MBean working and use new org.enableit.util.Log class
 *
 * Revision 1.2  2001/06/01 08:37:18  tstephen
 * lots of untracked changes leading to v1.0
 *
 * Revision 1.1.1.1  2001/04/23 05:27:12  tstephen
 * finally get under source control
 *
 */

// Package declaration 
package org.enableit.db ; 
 
// Java Imports
import java.io.InputStream ; 
import java.io.IOException ; 
import java.net.URL ; 
import java.sql.Connection ; 
import java.sql.DriverManager ; 
import java.sql.SQLException ; 
import java.util.ArrayList ; 
import java.util.Iterator ; 
import java.util.Properties ; 
import java.util.ResourceBundle ; 
import java.util.Set ; 

// Log4J Imports
import org.apache.log4j.Category;

/** 
 * <p>General class intended to hide the process of obtaining 
 * database connections to any database </p>
 * 
 * <p>The connection properties such as server name, username/password etc 
 * are all stored in a file named db.properties in the package level 
 * directory matching this class (ie <code>thestephensons.db</code>) </p>
 * 
 * <p>Internal methods then arrange these in the way to construct a 
 * Connection of the required type. Because this may vary from one vendor 
 * to the next, internally specific methods exist for each database driver
 * vendor</p>
 * 
 * <p>Supported database drivers currently are: </p>
 * <ul>
 *  <li>ASE/ASA (Sybase jConnect 5.x)</li>
 *  <li>MySQL (MM.MySQL driver)</li>
 * </ul>
 * 
 * @version v1.0
 * 
 * @author __AUTHOR__ 
 * 
 */
public class ConnectionFactory
{

/*
 * Constructors
 */
	/** 
	 * Default Constructor
	 */
	private ConnectionFactory() 
	{
		cat.debug("<ENTRY>") ;

        // Need to initialise with something, use props file in case no JNDI available
        props = new Properties() ;
        try {
            // Check class instance and get source properties
            InputStream is = this.getClass().getResourceAsStream("db.properties") ; 
            props.load(is) ; 
        } catch (IOException ioe) {
            cat.warn( "WARNING: Unable to read db connection properties: " + ioe.getMessage() ) ; 
            // throw new DBException("ERROR: Getting db connection properties: " + ioe.getMessage()) ; 
        }

	}

/*
 * Methods
 */

	/** 
	 * Factory method
	 */
	public static ConnectionFactory getInstance() 
	{ 
		cat.debug("<ENTRY>") ;

        if (me==null) me = new ConnectionFactory() ; 		
		return me ; 
	} 

	/** 
	 * Set a connection property dynamically
	 * 
	 */
	public static void setProperty(String key, String value) 
	{
		cat.debug("<ENTRY>") ;

                // Ensure we are initialised 
                getInstance() ; 
		props.setProperty(key,value) ; 
	}

    /** 
     * Returns a connection to the database identified by the 
     * class's resource properties 
     * 
     * @return Connection 
     * 
     * @throws DBException
     * in the event of a problem creating the connection 
     */
    public static Connection getConnection() 
    throws DBException
    { 
		cat.debug("<ENTRY>") ;

        Connection conn = null ; 
        try {
            conn = getInstance().getConnection(props) ; 
        } catch (DBException dbe) {
            // simply rethrow 
            throw dbe ; 
        } catch (Exception e) {
            cat.error( "ERROR: " + e.getMessage() ) ; 

            throw new DBException("ERROR: " + e.getMessage()) ; 
        }

        if (conn==null) {
            cat.error( "ERROR: Unable to create connection" ) ; 
            throw new DBException("ERROR: Unable to create connection: ") ; 
        }
        
        cat.debug("<EXIT>") ;
		return conn ;
    } 

    /** 
     * <p>Returns a connection to the database identified by the 
     * supplied properties</p>
     * <p>Parameters may be empty strings, but may not be null</p>
     * 
     * @param driver 
     * @param database 
     * @param server 
     * @param port 
     * @param protocol 
     * @param user 
     * @param password 
     * 
     * @return Connection 
     * 
     * @throws DBException
     * in the event of a problem creating the connection 
     */
    public static Connection getConnection( String driver, 
                                            String database, 
                                            String server, 
                                            String port, 
                                            String protocol, 
                                            String user, 
                                            String password ) 
    throws DBException
    { 
		cat.debug("<ENTRY>") ;

        Connection conn = null ; 
        try {
            // Check class instance and get propsource properties
            getInstance() ; 

			props.put("DRIVER", driver) ; 
            props.put("DATABASE_NAME", database) ; 
            props.put("SERVER_NAME", server) ; 
            props.put("PORT", port) ; 
            props.put("PROTOCOL", protocol) ; 
            props.put("USER", user) ; 
            props.put("PASSWORD", password) ; 

            conn = getConnection(props) ; 
        } catch (DBException dbe) {
            // simply rethrow 
            throw dbe ; 
        } catch (Exception e) {
            cat.error( "ERROR: " + e.getMessage() ) ; 

            throw new DBException("ERROR: " + e.getMessage()) ; 
        }

        if (conn==null) {
            cat.error( "ERROR: Unable to create connection" ) ; 
            throw new DBException("ERROR: Unable to create connection: ") ; 
        }
        
        cat.debug("<EXIT>") ;
		return conn ;
    } 

    /** 
     * Returns a connection to the database identified by the 
     * <code>Properties</code> supplied
     * 
     * @return Connection 
     * 
     * @throws DBException
     * in the event of a problem creating the connection 
     */
    protected static Connection getConnection( Properties props ) 
    throws DBException
    { 
		cat.debug("<ENTRY>") ;

        Connection conn = null ; 
        try {

            String driver = props.getProperty("DRIVER") ;
            if (driver.equals("org.gjt.mm.mysql.Driver")) {
                conn = getMySQLConnection(props) ; 
            } else if (driver.equals("com.sybase.jdbc2.jdbc.SybDriver") ) {
                conn = getSybConnection(props) ;             
            } else { 
                cat.error( "ERROR: Unsupported driver: " + driver ) ; 
                throw new DBException("ERROR: Unsupported driver: " + driver) ; 
            } 

        } catch (SQLException sqle) {
            cat.error( "ERROR: Getting connection: " + sqle.getMessage() ) ; 
            throw new DBException("ERROR: Getting connection: " + sqle.getMessage()) ; 
        } catch (Exception e) {
            cat.error( "ERROR: " + e.getMessage() ) ; 
            e.printStackTrace() ; 

            throw new DBException("ERROR: " + e.getMessage()) ; 
        }

        if (conn==null) {
            cat.error( "ERROR: Unable to create connection" ) ; 
            throw new DBException("ERROR: Unable to create connection: ") ; 
        }
        
        cat.debug("<EXIT>") ;
		return conn ;
    } 


    private static Connection getMySQLConnection( Properties props ) 
    throws java.sql.SQLException, DBException
    { 
		cat.debug("<ENTRY>") ;
        
        try {
            Class.forName(props.getProperty("DRIVER")).newInstance();         	
        } catch (IllegalAccessException iae) {
            cat.error( "ERROR: " + iae.getMessage() ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + iae.getMessage()) ; 
        } catch (ClassNotFoundException cnfe) {
            cat.error( "ERROR: " + cnfe.getMessage() ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + cnfe.getMessage()) ; 
        } catch (InstantiationException ie) {
            cat.error( "ERROR: " + ie.getMessage() ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + ie.getMessage()) ; 
        }

        String connectionString = "jdbc:" ;  
        connectionString += props.getProperty("PROTOCOL") ; 
        connectionString += "://" ; 
        connectionString += props.getProperty("SERVER_NAME") ; 
        connectionString += "/" ; 
        connectionString += props.getProperty("DATABASE_NAME") ; 
        connectionString += "?" ; 
        connectionString += "user=" ; 
        connectionString += props.getProperty("USER") ; 
        connectionString += "&" ; 
        connectionString += "password=" ; 
        connectionString += props.getProperty("PASSWORD") ; 

        cat.info("Connection string: " + connectionString);
        Connection conn = DriverManager.getConnection(connectionString);

        cat.debug("<EXIT>") ;
		return conn ;

    } 

    private static Connection getSybConnection( Properties props ) 
    throws java.sql.SQLException, DBException
    { 
		cat.debug("<ENTRY>") ;
        
        try {
            Class.forName(props.getProperty("DRIVER")).newInstance();         	
        } catch (IllegalAccessException iae) {
            cat.error( "ERROR: " + iae.getMessage() ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + iae.getMessage()) ; 
        } catch (ClassNotFoundException cnfe) {
            cat.error( "ERROR: " + cnfe.getMessage() ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + cnfe.getMessage()) ; 
        } catch (InstantiationException ie) {
            cat.error( "ERROR: " + ie.getMessage() ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + ie.getMessage()) ; 
        }

        String connectionString = "jdbc:sybase:" ;  
        connectionString += props.getProperty("PROTOCOL") ; 
        connectionString += ":" ; 
        connectionString += props.getProperty("SERVER_NAME") ; 
        connectionString += ":" ; 
        connectionString += props.getProperty("PORT") ; 
        Properties jcprops = new Properties() ; 
        jcprops.put("user", props.getProperty("USER")) ; 
        jcprops.put("password", props.getProperty("PASSWORD")) ; 

        cat.info("Connection string: " + connectionString);
        Connection conn = DriverManager.getConnection(connectionString, jcprops);

        cat.debug("<EXIT>") ;
		return conn ;

    } 

/*
 * Properties
 */

    /** 
     * The Log4J <code>Category</code> doing the logging.
     * Same <code>Category</code> is used throughout the package.
     */
    protected static Category cat = Category.getInstance("org.enableit.db");

    /** 
     * Reference to the singleton instance
     * 
     * NB the singleton status is not enforced via a synchronise statement
     */
    private static ConnectionFactory me ; 

	/** 
	 * The current database properties to make connections with
	 */
	private static Properties props ; 

    /** 
     * Information on the exact CVS version accessible after compilation
     */
    public static final String about = "$Revision: 1.3 $";

}
