package org.enableit.db;

/*
 * LIBRARY			: Database Integration
 *
 * COPYRIGHT        : Copyright (C) 2000 enableIT.org
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log$
 */

// Java Imports
import java.util.*;
import java.sql.*;
import javax.naming.*;

// J2EE extensions 
import javax.sql.*;

// Log4J Imports
import org.apache.log4j.Category;

/** 
 * Proxy class to wrap database access in a simple form. The user is required to 
 * provide their own <CODE>Connection</CODE>
 * 
 * @version v1.2
 * 
 * @author __AUTHOR__
 * 
 */
public class DataSourceProxy 
{

    /** 
     * Get a connection from the named DataSource. 
     * This method is intended for use by this class itself, but may be useful 
     * to other objects, in that case care must be taken to close the 
     * <code>Connection</code> properly. 
     * 
     * @param dataSourceName
     * The DataSource to lookup to obtain a connection 
     */
    public static Connection getConnection(String dataSourceName) 
    throws DBException     
    { 
		cat.debug("<ENTRY>");
        
        Connection conn = null ;
        try {
        	InitialContext ic = new InitialContext() ; 
            Context envContext = (Context)ic.lookup("java:/comp/env") ; 
            DataSource ds = (DataSource)envContext.lookup(dataSourceName) ;
            conn = ds.getConnection() ; 
        } catch (Exception e) {
            cat.error(e) ; 
            throw new DBException(e.getMessage()) ;
        }

		cat.debug("<EXIT>");
        return conn ;
    } 

    /**
     * Execute the supplied SQL query against a connection 
	 * obtained from the named DataSource.
	 *
	 * @param dataSourceName
	 * The datasource name to lookup in order to get a connection 
	 *
	 * @param sql
	 * The SQL query to execute
	 *
	 * @return
	 * The SQL results stored as an <code>java.util.ArrayList</code>, each 
     * element of which is a <code>java.util.TreeMap</code> holding a 
     * single row of results
	 * 
	 * @throws DBException
	 * If execution of the query failed.
     */
    public static java.util.List executeQuery(String dataSourceName, String sql) 
    throws DBException
    { 
		cat.debug("<ENTRY>");
        
        ArrayList results = null ; 
      	Connection conn = null ; 
        try {
            conn = DataSourceProxy.getConnection(dataSourceName) ; 
            results = DatabaseProxy.executeQuery(conn, sql) ; 
        } catch (Exception e) {
            cat.error(e) ; 
            throw new DBException(e.getMessage()) ; 
        } finally { 
            try {
                conn.close(); 
                conn = null ;
            } catch (Exception e) {
                cat.fatal(e) ;             
            }
        } 

		cat.debug("<EXIT>");
        return results ;
    } 

    /**
     * Execute the supplied SQL update against a the supplied
	 * database connection.
	 *
	 * @param sql
	 * The SQL update to execute
	 * 
	 * @param dataSourceName
	 * The datasource name to lookup in order to get a connection 
	 *
	 * @return
	 * The number of rows affected by the update.
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static int executeUpdate(String dataSourceName, String sql) 
    throws DBException
    {
		cat.debug("<ENTRY>");
        
        int rowsAffected = 0 ; 
      	Connection conn = null ; 
        try {
            conn = DataSourceProxy.getConnection(dataSourceName) ; 
            rowsAffected = DatabaseProxy.executeUpdate(conn, sql) ; 
        } catch (Exception e) {
            cat.error(e) ; 
            throw new DBException(e.getMessage()) ; 
        } finally { 
            try {
                conn.close(); 
                conn = null ;
            } catch (Exception e) {
                cat.fatal(e) ;             
            }
        } 

		cat.debug("<EXIT>");
        return rowsAffected;
    }

    /**
     * Execute the supplied SQL update against a connection 
	 * obtained from the named DataSource.
	 *
	 * @param dataSourceName
	 * The datasource name to lookup in order to get a connection 
	 *
	 * @param sql
	 * The SQL update to execute with parameters represented by ? 
     * 
	 * @param parms
	 * <CODE>ArrayList</CODE> containing parameters to insert in the sql statement
	 * 
	 * @return
	 * The number of rows affected by the update.
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static int executeUpdate(String dataSourceName, String sql, List parms) 
    throws DBException
    {
		cat.debug("<ENTRY>");

        int rowsAffected = 0 ; 
      	Connection conn = null ; 
        try {
            conn = DataSourceProxy.getConnection(dataSourceName) ; 
            rowsAffected = DatabaseProxy.executeUpdate(conn, sql, parms) ; 
        } catch (Exception e) {
            cat.error(e) ; 
            throw new DBException(e.getMessage()) ; 
        } finally { 
            try {
                conn.close(); 
                conn = null ;
            } catch (Exception e) {
                cat.fatal(e) ;             
            }
        } 

		cat.debug("<EXIT>");
		return rowsAffected;
    }

    /**
     * Execute the supplied update procedure against a connection 
	 * obtained from the named DataSource.
	 *
	 * @param dataSourceName
	 * The datasource name to lookup in order to get a connection 
	 *
	 * @param sp
	 * A string containing the stored procedure name 
	 * 
	 * @param parms
	 * A <code>java.util.List</code> of String parameters for the stored procedure
	 *
	 * @return
	 * The number of rows affected by the stored procedure executed
	 *
	 * @throws DBException
	 * If execution of the stored procedure failed.
     */
    public static int executeDmlProcedure(String dataSourceName, String sp, List parms) 
    throws DBException
    {
		cat.debug("<ENTRY>");
        
        int rowsAffected = 0 ; 
      	Connection conn = null ; 
        try {
            conn = DataSourceProxy.getConnection(dataSourceName) ; 
            rowsAffected = DatabaseProxy.executeUpdate(conn, sp, parms) ; 
        } catch (Exception e) {
            cat.error(e) ; 
            throw new DBException(e.getMessage()) ; 
        } finally { 
            try {
                conn.close(); 
                conn = null ;
            } catch (Exception e) {
                cat.fatal(e) ;             
            }
        } 

		cat.debug("<EXIT>");
		return rowsAffected ;
    }

    /**
     * Execute the supplied query procedure against a connection 
	 * obtained from the named DataSource.
	 *
	 * @param dataSourceName
	 * The datasource name to lookup in order to get a connection 
	 *
	 * @param sp
	 * A string containing the stored procedure name 
	 * 
	 * @param parms
	 * A <code>java.util.List</code> of String parameters for the stored procedure
     * 
	 * @return
	 * The SQL results stored as an <code>java.util.ArrayList</code>, each 
     * element of which is a <code>java.util.TreeMap</code> holding a 
     * single row of results
	 *
	 * @throws DBException
	 * If execution of the stored procedure failed.
     */
    public static java.util.List executeQueryProcedure(String dataSourceName, String sp, List parms) 
    throws DBException
    {
		cat.debug("<ENTRY>");

        List results = null ; 
      	Connection conn = null ; 
        try {
            conn = DataSourceProxy.getConnection(dataSourceName) ; 
            results = (List)DatabaseProxy.executeQueryProcedure(conn, sp, parms, DatabaseProxy.LIST) ; 
        } catch (Exception e) {
            cat.error(e) ; 
            throw new DBException(e.getMessage()) ; 
        } finally { 
            try {
                conn.close(); 
                conn = null ;
            } catch (Exception e) {
                cat.fatal(e) ;             
            }
        } 

		cat.debug("<EXIT>");
		return results ;
	} 


    /** 
     * The Log4J <code>Category</code> doing the logging.
     * Same <code>Category</code> is used throughout the library.
     */
    protected static Category cat = Category.getInstance("org.enableit.db");

	/**  
	 * CVS info about this class and its current version
	 */
	public static final String about = "$Id$" ;

}

