package org.enableit.db;

/*
 * LIBRARY			: __PROJECT_NAME__
 *
 * COPYRIGHT        : Copyright (C) 2000 enableIT.org
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: DatabaseProxy.java,v $
 * Revision 1.2  2001/12/03 12:42:31  __CVS_AUTHOR__
 * changes to get the MBean working and use new org.enableit.util.Log class
 *
 * Revision 1.1.1.1  2001/04/23 05:27:12  __CVS_AUTHOR__
 * finally get under source control
 *
 */

// Java Imports
import java.util.*;
import java.sql.*;

// Log4J Imports
import org.apache.log4j.Category;

/** 
 * Proxy class to wrap database access in a simple form. The user is required to 
 * provide their own <CODE>Connection</CODE>
 * 
 * @version v1.0
 * 
 * @author __AUTHOR__ 
 * 
 */
public class DatabaseProxy 
{
    
    /**
     * Execute the supplied SQL query against the supplied database 
	 * connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sql
	 * The SQL query to execute
	 *
	 * @return
	 * The SQL results stored as an <code>java.util.ArrayList</code>, each 
     * element of which is a <code>java.util.TreeMap</code> holding a 
     * single row of results
	 * 
	 * @throws DBException
	 * If execution of the query failed.
     */
    public static java.util.ArrayList executeQuery(Connection conn, String sql) 
    throws DBException
    { 
        return (java.util.ArrayList)executeQuery(conn, sql, 1) ; 
    } 

    
    /**
     * Execute the supplied SQL query against the supplied database 
	 * connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sql
	 * The SQL query to execute
	 *
	 * @param returnType
	 * The object type to convert the SQL results into 
	 *
	 * @return
	 * The SQL results as the object type requested
	 * 
	 * @throws DBException
	 * If execution of the query failed.
     * 
     * @deprecated <br/>
     * use <code>executeQuery(conn:Connection, sql:String):java.util.ArrayList</code> instead
     */
    public static Object executeQuery(Connection conn, String sql, int returnType) 
    throws DBException
    {
		cat.debug("<ENTRY>");

		Object o = null;
		Statement stmt			= null;
		java.sql.ResultSet rs	= null ;

        try {
			// create a statement on the connection
			stmt=conn.createStatement();

			// if appropriate write out trace information before executing SQL
			cat.debug("<SQL_SUMMARY> About to execute SQL query: " + sql);
			
			// Do the query 
			boolean results=stmt.execute(sql);
			
			cat.debug("<SQL_RESULT> Results found ?: " + String.valueOf(results));

			// If one exists get the result set
			if ( results ) {
				rs = stmt.getResultSet() ; 
				// Do any object conversions required 
				switch (returnType) {
					case 1:
						List resultList = DBUtils.convertResultToList(rs) ; 					
						o = resultList; 
						break;
					case 2:
                        throw new DBException("Attempt to use deprecated functionality that has been compiled out of this version") ; 
                    default :
						o = rs ; 
				}
			}

			// Print any warnings chained to statement object
            SQLWarning warning = stmt.getWarnings();
            while( warning != null ){
            	cat.warn("<SQL Warning>" + warning.toString());
            	warning = warning.getNextWarning() ;
            }
            // Print any warnings chained to ResultSet object
            warning = rs.getWarnings();
            while( warning != null ){
                cat.debug("<SQL Warning>" + warning.toString());
                warning = warning.getNextWarning() ;
            }
		} catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			cat.error("\n ==========  Database error encountered code  = " + String.valueOf(errCode) + "==========");
            cat.error(sqle);
			cat.error("SQL executed was: " + sql);
			throw new DBException(sqle.getMessage());
		} catch (Exception e) {
			cat.fatal(e.getMessage());
			throw new DBException(e.getMessage());
		} finally {
			try { 
				stmt.close(); 
				stmt=null; 
			} catch (Exception e) {
				cat.fatal(e.getMessage()); 
			}
		}

		cat.debug("<EXIT>");
		return o ; 
    }

    /**
     * Execute the supplied SQL update query against a the supplied
	 * database connection.
	 *
	 * @param sql
	 * The SQL update to execute
	 * 
	 * @param conn
	 * The database connection to use
	 *
	 * @return
	 * The number of rows affected by the update.
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static int executeUpdate(Connection conn, String sql) 
    throws DBException
    {
		cat.debug("<ENTRY>");
        
        int rowsAffected            = 0;
		Statement stmt 	            = null;

		try {
			// Create a JDBC connection and execute the String
			stmt = conn.createStatement();

			//  If appropriate write out trace information before executing
			cat.debug("<SQL_SUMMARY> About to execute SQL: \n" + sql);
			rowsAffected = stmt.executeUpdate(sql);

			// Print any warnings chained to statement object
			SQLWarning warning = stmt.getWarnings();
			while( warning != null ){
				cat.debug("<SQL Warning :>" + warning.toString());
				warning = warning.getNextWarning() ;
			}

		} catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			cat.error("\n ==========  Database error encountered code  = " + String.valueOf(errCode) + "==========");
            cat.error(sqle);
			cat.error("SQL executed was: " + sql);
			throw new DBException(sqle.getMessage());
		} catch (Exception e) {
			cat.fatal(e.getMessage());
			throw new DBException(e.getMessage());
		} finally {
			try { stmt.close(); } catch (Exception e) {}
		}

		cat.debug("<EXIT>");
        return rowsAffected;
    }

    /**
     * Execute the supplied SQL update query against a the supplied
	 * database connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sql
	 * The SQL update to execute with parameters represented by ? 
	 * 
	 * @param parms
	 * <CODE>ArrayList</CODE> containing parameters to insert in the sql statement
	 * 
	 * @return
	 * The number of rows affected by the update.
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static int executeUpdate(Connection conn, String sql, List parms) throws SQLException, DBException
    {
		cat.debug("<ENTRY>");

		int rowsAffected            = 0;
		PreparedStatement ps 	            = null;

		try {
			// Create a JDBC connection and execute the String
			ps = conn.prepareStatement(sql);
			if (parms ==null ) {
				parms = new ArrayList();
			}
			// Set the parameters
			for ( int j=0 ; j<parms.size() ;j++ ) {
				if ( parms.get(j) == null ) {	
					ps.setNull( j+1, Types.NULL );
				} else if ( parms.get(j) instanceof java.lang.Long ) {
					ps.setLong( j+1, ((java.lang.Long)parms.get(j)).longValue() );
				} else if ( parms.get(j) instanceof java.lang.String ) {
					ps.setString( j+1, (java.lang.String)parms.get(j) );
				} else {
					cat.debug("Unknown parameter type supplied for UPDATE") ; 
					throw new DBException ("Unknown parameter type supplied for UPDATE") ; 
				}
			} 

			//  If appropriate write out trace information before executing
			cat.debug("<SQL_SUMMARY> About to execute SQL: \n" + sql);
            cat.debug("<SQL_VERBOSE> ... with parameters: ") ; 
            for ( int j=0 ; j<parms.size() ;j++ ) {
                cat.debug( "<SQL_VERBOSE> ... parm" + String.valueOf(j+1) + ": " + parms.get(j).toString()) ; 
            }
			rowsAffected = ps.executeUpdate();

			// Print any warnings chained to statement object
			SQLWarning warning = ps.getWarnings();
			while( warning != null ){
				cat.warn("<SQL Warning :>" + warning.toString());
				warning = warning.getNextWarning() ;
			}

		} 
		catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			cat.error("\n ==========  Database error encountered code  = " + String.valueOf(errCode) + "==========");
            cat.error(sqle);
			cat.error("SQL executed was: " + sql);
			throw new DBException(sqle.getMessage());
		} 
		catch (Exception e) {
			cat.fatal(e.getMessage());
			throw new DBException(e.getMessage());
		} 
		finally {
			try { ps.close(); } catch (Exception e) {}
		}

		cat.debug("<EXIT>");
		return rowsAffected;
    }
    /**
     * Execute the supplied stored procedure against the supplied 
	 * database connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sp
	 * A string containing the stored procedure name 
	 * 
	 * @param parms
	 * A <code>java.util.List</code> of String parameters for the stored procedure
	 *
	 * @return
	 * The number of rows affected by the stored procedure executed
	 *
	 * @throws DBException
	 * If execution of the stored procedure failed.
     */
    public static int executeDmlProcedure(Connection conn, String sp, List parms) 
    throws DBException
    {
		cat.debug("<ENTRY>");
        
        List resultList             = null;
		CallableStatement cs        = null;
		int i ; 

        try {
			int j ; 

			StringBuffer spCall = new StringBuffer( "{call " + sp + "(" ) ;

			if (parms ==null ) {
				parms = new ArrayList();
			}

			for ( j=0 ; j<parms.size() ;j++ ) {
				spCall.append("?,") ; 
			}
			// remove the trailing comma
			StringBuffer spCall2 = new StringBuffer( spCall.substring(0, spCall.length()-1) ) ; 
			spCall2.append(")}") ; 
			cs = conn.prepareCall( spCall2.toString() );

			for ( j=0 ; j<parms.size() ;j++ ) {
				if ( parms.get(j) instanceof java.lang.Long ) {
					cs.setLong( j+1, ((java.lang.Long)parms.get(j)).longValue() );
				} else if ( parms.get(j) instanceof java.lang.String ) {
					cs.setString( j+1, (java.lang.String)parms.get(j) );
				} else if ( parms.get(j) instanceof Integer ) {
					cs.setInt( j+1, ((Integer)parms.get(j)).intValue() );
                } else if ( parms.get(j) instanceof Object ) {
                    // This may not work depending on the RDBMS handling of Objects
					cs.setObject( j+1, parms.get(j) );
				} else {
					cat.debug("Unknown parameter type supplied for stored procedure") ; 
					throw new DBException ("Unknown parameter type supplied for stored procedure") ; 
				}
			} 

			// if appropriate write out trace information before executing SQL
			cat.debug("<SQL_SUMMARY> About to execute stored procedure: " + sp);
			cat.debug("<SQL_VERBOSE> ... with parameters: ") ; 
            for ( j=0 ; j<parms.size() ;j++ ) {
                if (parms.get(j)!=null) {
                    cat.debug( "<SQL_VERBOSE> ... " + parms.get(j).getClass().getName() + " parm" + String.valueOf(j+1) + ": " + 
                                                      parms.get(j).toString()) ; 
                } else {
                    cat.debug( "<SQL_VERBOSE> ... parm" + String.valueOf(j+1) + ": null") ; 
                }
            }

            i=cs.executeUpdate();

			// Print any warnings chained to statement object
            SQLWarning warning = cs.getWarnings();
            while( warning != null ) {
            	cat.warn("<SQL Warning>" + warning.toString());
            	warning = warning.getNextWarning() ;
            }

		}
		catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			cat.error("==========  Database error encountered: code  = " + String.valueOf(errCode) + " ==========");
            cat.error(sqle);
			cat.error("Stored procedure executed was: " + sp);
			throw new DBException(sqle.getMessage());
		} catch (Exception e) {
			cat.fatal(e.getMessage());
			throw new DBException(e.getMessage());
		} finally {
			try { 
				cs.close() ; 
				cs = null ;
			} catch (Exception e) {}
		}

		cat.debug("<EXIT>");
		return i ;
    }

    /**
     * Execute the supplied stored procedure against the supplied 
	 * database connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sp
	 * A string containing the stored procedure name 
	 * 
	 * @param parms
	 * A <code>java.util.List</code> of String parameters for the stored procedure
	 *
     * @param returnType
     * One of the defined return types defined as constants within this class.
     * 
	 * @return
	 * <code>java.lang.Object</code> containing any rows found. 
     * The <code>Object</code> type is defined by the return type parameter
	 *
	 * @throws DBException
	 * If execution of the stored procedure failed.
     */
    public static java.lang.Object executeQueryProcedure(Connection conn, String sp, List parms, int returnType) throws DBException
    {
		cat.debug("<ENTRY>");

		java.lang.Object o		= null ;
		//List resultList         = null;
		CallableStatement cs    = null;
		boolean results			= false ; 
		java.sql.ResultSet rs	= null ; 

        try {
			int j ; 

			StringBuffer spCall = new StringBuffer( "{?=call " + sp + "(" ) ;
			if (parms ==null ) {
				parms = new ArrayList();
			}
			for ( j=0 ; j<parms.size() ;j++ ) {
				spCall.append("?,") ; 
			}
			// remove the trailing comma
			StringBuffer spCall2 = new StringBuffer( spCall.substring(0, spCall.length()-1) ) ; 
			spCall2.append(")}") ; 

			cat.debug( spCall2.toString() ) ; 
			cs = conn.prepareCall( spCall2.toString() );

			for ( j=0 ; j<parms.size() ;j++ ) {
				if ( parms.get(j) instanceof java.lang.String ) {
					cs.setString( j+2, (java.lang.String)parms.get(j) );
				} else if ( parms.get(j) instanceof java.lang.Long ) {
					cs.setLong( j+2, ((java.lang.Long)parms.get(j)).longValue() );
				} else {
					cat.debug("Unknown parameter type supplied for stored procedure") ; 
					throw new DBException ("Unknown parameter type supplied for stored procedure") ; 
				}
			} 

			// if appropriate write out trace information before executing SQL
			cat.debug("<SQL_SUMMARY> About to execute stored procedure: " + sp);

            cat.debug("<SQL_VERBOSE> ... with parameters: ") ; 
            for ( j=0 ; j<parms.size() ;j++ ) {
                cat.debug( "<SQL_VERBOSE> ... parm" + String.valueOf(j+1) + ": " + parms.get(j).toString()) ; 
            }

			rs=cs.executeQuery();
			
			if ( rs == null ) {
				cat.debug("<SQL_VERBOSE> failed to get result set object "); 
			}
			// Do any object conversions required 
			switch (returnType) {
				case 1:
					List resultList = DBUtils.convertResultToList(rs) ; 					
					o = resultList; 
					break;
                case 2:
                    throw new DBException("Attempt to use deprecated functionality that has been compiled out of this version") ; 
				default :
					o = rs ; 
			}

			// Print any warnings chained to statement object
            SQLWarning warning = cs.getWarnings();
            while( warning != null ) {
            	cat.warn("<SQL Warning>" + warning.toString());
            	warning = warning.getNextWarning() ;
            }

		}
		catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			cat.error("==========  Database error encountered: code  = " + String.valueOf(errCode) + " ==========");
            cat.error(sqle);
			cat.error("Stored procedure executed was: " + sp);
			throw new DBException(sqle.getMessage());
		}
		catch (Exception e) {
			cat.fatal(e.getMessage());
			throw new DBException(e.getMessage());
		} 
		finally {
			try { cs.close(); } catch (Exception e) {}
		}

		cat.debug("<EXIT>");
		return o ;
	} 

/* 
 * Return types that may be used by the query methods
 */
	/** Valid Object type for result set returns */
	public static final int LIST = 1; 

	/** 
     * Valid Object type for result set returns. 
     * When returning references to a <code>java.sql.ResultSet</code> the developer 
     * must take care to ensure that the reference and the <code>Connection</code>
     * that created it are correctly closed to avoid a memory leak. 
     */
	public static final int SQL_RESULT_SET = 3; 

    /** 
     * The Log4J <code>Category</code> doing the logging.
     * Same <code>Category</code> is used throughout the library.
     */
    protected static Category cat = Category.getInstance("org.enableit.db");

	/**  
	 * CVS info about this class and its current version
	 */
	public static final String about = "$Id$" ;

}

