/*
 * PROJECT			: Database integration library
 *
 * COPYRIGHT        : Copyright (C) 2000 Tim Stephenson
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: ConnectionFactory.java,v $
 * Revision 1.3  2001/12/03 12:42:31  tstephen
 * changes to get the MBean working and use new org.enableit.util.Log class
 *
 * Revision 1.2  2001/06/01 08:37:18  tstephen
 * lots of untracked changes leading to v1.0
 *
 * Revision 1.1.1.1  2001/04/23 05:27:12  tstephen
 * finally get under source control
 *
 */

// Package declaration 
package org.thestephensons.db ; 
 
// Java Imports
import java.io.InputStream ; 
import java.io.IOException ; 
import java.net.URL ; 
import java.sql.Connection ; 
import java.sql.DriverManager ; 
import java.sql.SQLException ; 
import java.util.ArrayList ; 
import java.util.Iterator ; 
import java.util.Properties ; 
import java.util.ResourceBundle ; 
import java.util.Set ; 

// Other extensions 
import javax.management.* ; 

// My Imports 
import org.enableit.util.Log ; 
import org.enableit.jmxapi.RepositoryObj ; 

/** 
 * <p>General class intended to hide the process of obtaining 
 * database connections to any database </p>
 * 
 * <p>The connection properties such as server name, username/password etc 
 * are all stored in a file named db.properties in the package level 
 * directory matching this class (ie <code>thestephensons.db</code>) </p>
 * 
 * <p>Internal methods then arrange these in the way to construct a 
 * Connection of the required type. Because this may vary from one vendor 
 * to the next, internally specific methods exist for each database driver
 * vendor</p>
 * 
 * <p>Supported database drivers currently are: </p>
 * <ul>
 *  <li>ASE/ASA (Sybase jConnect 5.x)</li>
 *  <li>MySQL (MM.MySQL driver)</li>
 * </ul>
 * 
 * @version __VERSION__
 * 
 * @author tim.stephenson@sybase.com 
 * 
 */
public class ConnectionFactory
{

/*
 * Constructors
 */
	/** 
	 * Default Constructor
	 */
	private ConnectionFactory() 
	{
		Log.write("ENTRY:", true, Log.METHOD_ENTRY);

        // Need to initialise with something, use props file in case no JNDI available
        props = new Properties() ;
        try {
            // Check class instance and get source properties
            InputStream is = this.getClass().getResourceAsStream("db.properties") ; 
            props.load(is) ; 
        } catch (IOException ioe) {
            Log.write( "WARNING: Unable to read db connection properties: " + ioe.getMessage(), true, Log.VERBOSE ) ; 
            // throw new DBException("ERROR: Getting db connection properties: " + ioe.getMessage()) ; 
        }

        // Now register my MBean for management purposes
        try {
            // Try to find MBean Server
            ArrayList servers = MBeanServerFactory.findMBeanServer(null) ; 

            // Register with all servers found
            MBeanServer mbs = null ; 
            for ( Iterator i=servers.iterator() ; i.hasNext() ; ) {
                mbs = (MBeanServer)i.next() ; 

                // Register with the server 
                Log.write("Registering ConnectionFactoryMBean with MBeanServer", true, Log.VERBOSE) ; 
                RepositoryObj ro = new RepositoryObj(mbs) ; 
                ObjectInstance oi = ro.register("org.thestephensons.db.ConnectionFactoryMBean") ; 
            }
        } catch (Exception e) {
            Log.write("Error whilst registering MBean", true, Log.VERBOSE) ;         
            Log.write(e) ;         
            e.printStackTrace() ; 
        }
	}

/*
 * Methods
 */

	/** 
	 * Factory method
	 */
	public static ConnectionFactory getInstance() 
	{ 
		Log.write("ENTRY:", true, Log.METHOD_ENTRY);

        if (me==null) me = new ConnectionFactory() ; 		
		return me ; 
	} 

	/** 
	 * Set a connection property dynamically
	 * 
	 */
	public static void setProperty(String key, String value) 
	{
		Log.write("ENTRY:", true, Log.METHOD_ENTRY);

                // Ensure we are initialised 
                getInstance() ; 
		props.setProperty(key,value) ; 
	}

    /** 
     * Returns a connection to the database identified by the 
     * class's resource properties 
     * 
     * @return Connection 
     * 
     * @throws DBException
     * in the event of a problem creating the connection 
     */
    public static Connection getConnection() 
    throws DBException
    { 
		Log.write("ENTRY:", true, Log.METHOD_ENTRY);

        Connection conn = null ; 
        try {
            conn = getInstance().getConnection(props) ; 
        } catch (DBException dbe) {
            // simply rethrow 
            throw dbe ; 
        } catch (Exception e) {
            Log.write( "ERROR: " + e.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            Log.write( e ) ; 

            throw new DBException("ERROR: " + e.getMessage()) ; 
        }

        if (conn==null) {
            Log.write( "ERROR: Unable to create connection", true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Unable to create connection: ") ; 
        }
        
        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return conn ;
    } 

    /** 
     * <p>Returns a connection to the database identified by the 
     * supplied properties</p>
     * <p>Parameters may be empty strings, but may not be null</p>
     * 
     * @param driver 
     * @param database 
     * @param server 
     * @param port 
     * @param protocol 
     * @param user 
     * @param password 
     * 
     * @return Connection 
     * 
     * @throws DBException
     * in the event of a problem creating the connection 
     */
    public static Connection getConnection( String driver, 
                                            String database, 
                                            String server, 
                                            String port, 
                                            String protocol, 
                                            String user, 
                                            String password ) 
    throws DBException
    { 
		Log.write("ENTRY:", true, Log.METHOD_ENTRY);

        Connection conn = null ; 
        try {
            // Check class instance and get propsource properties
            getInstance() ; 

			props.put("DRIVER", driver) ; 
            props.put("DATABASE_NAME", database) ; 
            props.put("SERVER_NAME", server) ; 
            props.put("PORT", port) ; 
            props.put("PROTOCOL", protocol) ; 
            props.put("USER", user) ; 
            props.put("PASSWORD", password) ; 

            conn = getConnection(props) ; 
        } catch (DBException dbe) {
            // simply rethrow 
            throw dbe ; 
        } catch (Exception e) {
            Log.write( "ERROR: " + e.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            Log.write( e ) ; 

            throw new DBException("ERROR: " + e.getMessage()) ; 
        }

        if (conn==null) {
            Log.write( "ERROR: Unable to create connection", true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Unable to create connection: ") ; 
        }
        
        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return conn ;
    } 

    /** 
     * Returns a connection to the database identified by the 
     * <code>Properties</code> supplied
     * 
     * @return Connection 
     * 
     * @throws DBException
     * in the event of a problem creating the connection 
     */
    protected static Connection getConnection( Properties props ) 
    throws DBException
    { 
		Log.write("ENTRY:", true, Log.METHOD_ENTRY);

        Connection conn = null ; 
        try {

            String driver = props.getProperty("DRIVER") ;
            if (driver.equals("org.gjt.mm.mysql.Driver")) {
                conn = getMySQLConnection(props) ; 
            } else if (driver.equals("com.sybase.jdbc2.jdbc.SybDriver") ) {
                conn = getSybConnection(props) ;             
            } else { 
                Log.write( "ERROR: Unsupported driver: " + driver, true, Log.CRITICAL_ERROR ) ; 
                throw new DBException("ERROR: Unsupported driver: " + driver) ; 
            } 

        } catch (SQLException sqle) {
            Log.write( "ERROR: Getting connection: " + sqle.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Getting connection: " + sqle.getMessage()) ; 
        } catch (Exception e) {
            Log.write( "ERROR: " + e.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            Log.write( e ) ; 
            e.printStackTrace() ; 

            throw new DBException("ERROR: " + e.getMessage()) ; 
        }

        if (conn==null) {
            Log.write( "ERROR: Unable to create connection", true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Unable to create connection: ") ; 
        }
        
        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return conn ;
    } 


    private static Connection getMySQLConnection( Properties props ) 
    throws java.sql.SQLException, DBException
    { 
		Log.write("ENTRY:", true, Log.METHOD_ENTRY);
        
        try {
            Class.forName(props.getProperty("DRIVER")).newInstance();         	
        } catch (IllegalAccessException iae) {
            Log.write( "ERROR: " + iae.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + iae.getMessage()) ; 
        } catch (ClassNotFoundException cnfe) {
            Log.write( "ERROR: " + cnfe.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + cnfe.getMessage()) ; 
        } catch (InstantiationException ie) {
            Log.write( "ERROR: " + ie.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + ie.getMessage()) ; 
        }

        String connectionString = "jdbc:" ;  
        connectionString += props.getProperty("PROTOCOL") ; 
        connectionString += "://" ; 
        connectionString += props.getProperty("SERVER_NAME") ; 
        connectionString += "/" ; 
        connectionString += props.getProperty("DATABASE_NAME") ; 
        connectionString += "?" ; 
        connectionString += "user=" ; 
        connectionString += props.getProperty("USER") ; 
        connectionString += "&" ; 
        connectionString += "password=" ; 
        connectionString += props.getProperty("PASSWORD") ; 

        Log.write("SQL_VERBOSE: connection string: " + connectionString, true, Log.SQL_VERBOSE);
        Connection conn = DriverManager.getConnection(connectionString);

        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return conn ;

    } 

    private static Connection getSybConnection( Properties props ) 
    throws java.sql.SQLException, DBException
    { 
		Log.write("ENTRY:", true, Log.METHOD_ENTRY);
        
        try {
            Class.forName(props.getProperty("DRIVER")).newInstance();         	
        } catch (IllegalAccessException iae) {
            Log.write( "ERROR: " + iae.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + iae.getMessage()) ; 
        } catch (ClassNotFoundException cnfe) {
            Log.write( "ERROR: " + cnfe.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + cnfe.getMessage()) ; 
        } catch (InstantiationException ie) {
            Log.write( "ERROR: " + ie.getMessage(), true, Log.CRITICAL_ERROR ) ; 
            throw new DBException("ERROR: Getting db connection properties: " + ie.getMessage()) ; 
        }

        String connectionString = "jdbc:sybase:" ;  
        connectionString += props.getProperty("PROTOCOL") ; 
        connectionString += ":" ; 
        connectionString += props.getProperty("SERVER_NAME") ; 
        connectionString += ":" ; 
        connectionString += props.getProperty("PORT") ; 
        Properties jcprops = new Properties() ; 
        jcprops.put("user", props.getProperty("USER")) ; 
        jcprops.put("password", props.getProperty("PASSWORD")) ; 

        Log.write("SQL_VERBOSE: connection string: " + connectionString, true, Log.VERBOSE);
        Connection conn = DriverManager.getConnection(connectionString, jcprops);

        Log.write("EXIT:", true, Log.METHOD_EXIT);
		return conn ;

    } 

/*
 * Properties
 */

    /** 
     * Reference to the singleton instance
     * 
     * NB the singleton status is not enforced via a synchronise statement
     */
    private static ConnectionFactory me ; 

	/** 
	 * The current database properties to make connections with
	 */
	private static Properties props ; 

    /** 
     * Information on the exact CVS version accessible after compilation
     */
    public static final String about = "$Revision: 1.3 $";

}
