package org.thestephensons.db;

/*
 * LIBRARY			: Database integration library
 *
 * COPYRIGHT        : Copyright (C) 2000 Tim Stephenson
 *
 * Change History:
 * -------------------------------------------------------------------
 * $Log: DatabaseProxy.java,v $
 * Revision 1.2  2001/12/03 12:42:31  tstephen
 * changes to get the MBean working and use new org.enableit.util.Log class
 *
 * Revision 1.1.1.1  2001/04/23 05:27:12  tstephen
 * finally get under source control
 *
 */

// Java Imports
import java.util.*;
import java.sql.*;

// Sybase Jaguar imports
//import com.sybase.jaguar.sql.*; 
//import com.sybase.jaguar.server.*; 
//import com.sybase.jaguar.jcm.*; 
//import com.sybase.jaguar.util.*; 
//import com.sybase.CORBA.jdbc11.*; 
//import TabularResults.ResultSet ; 

// My imports
import org.enableit.util.Log ;

/** 
 * Proxy class to wrap database access in a simple form. The user is required to 
 * provide their own <CODE>Connection</CODE>
 * 
 * 
 * @version __VERSION__
 * 
 * @author tim.stephenson@sybase.com 
 * 
 */
public class DatabaseProxy 
{
    /**
     * Execute the supplied SQL query against the supplied database 
	 * connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sql
	 * The SQL query to execute
	 *
	 * @param returnType
	 * The object type to convert the SQL results into 
	 *
	 * @return Object
	 * The SQL results as the object type requested
	 * 
	 * @throws DBException
	 * If execution of the query failed.
     */
    public static Object executeQuery(Connection conn, String sql, int returnType) throws DBException
    {
		if (Log.METHOD_ENTRY) {
			Log.write("<ENTRY>",true);
		}

		Object o = null;
		//List result = null ; 
		//TabularResults.ResultSet result = null ; 
		Statement stmt			= null;
		java.sql.ResultSet rs	= null ;

        try {
			// create a statement on the connection
			stmt=conn.createStatement();

			// if appropriate write out trace information before executing SQL
			if ( Log.SQL_SUMMARY ) {
				Log.write("<SQL_SUMMARY> About to execute SQL query: " + sql, true);
			}
			
			// Do the query 
			boolean results=stmt.execute(sql);
			
			if ( Log.SQL_RESULT ) {
				Log.write("<SQL_RESULT> Results found ?: " + String.valueOf(results), true);
			}

			// If one exists get the result set
			if ( results ) {
				rs = stmt.getResultSet() ; 
				// Do any object conversions required 
				switch (returnType) {
					case 1:
						List resultList = convertResultToList(rs) ; 					
						o = resultList; 
						break;
					case 2:
/* Sybase proprietary functionality 
                        TabularResults.ResultSet resultTRS = null ; 
						try {
							resultTRS = IDL.getResultSet(rs) ; 					
						}
						catch (SQLException e) {
							Log.write("Conversion to Tabular Result Set fails", true); 
						}
						o = resultTRS ; 
*/
                        throw new DBException("Attempt to use Sybase proprietary technology that has been compiled out of this version") ; 
                    default :
						o = rs ; 
				}


			}

			// Print any warnings chained to statement object
            SQLWarning warning = stmt.getWarnings();
            while( warning != null ){
            	Log.write("<SQL Warning>" + warning.toString());
            	warning = warning.getNextWarning() ;
            }
            // Print any warnings chained to ResultSet object
            warning = rs.getWarnings();
            while( warning != null ){
                Log.write("<SQL Warning>" + warning.toString());
                warning = warning.getNextWarning() ;
            }

		}
		catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			Log.write("\n ==========  Database error encountered code  = " + String.valueOf(errCode) + "==========");
            Log.write(sqle);
			Log.write("SQL executed was: " + sql);
			throw new DBException(sqle.getMessage());
		}
		catch (Exception e) {
			Log.write(e.getMessage());
			throw new DBException(e.getMessage());
		} 
		finally {
			try { 
				stmt.close(); 
				stmt=null; 
			} 
			catch (Exception e) {
				Log.write(e.getMessage()); 
			}
		}

		if (Log.METHOD_EXIT) {
			Log.write("<EXIT>",true);
		}

		return o ; 
    }
    /**
     * Execute the supplied SQL update query against a the supplied
	 * database connection.
	 *
	 * @param sql
	 * The SQL update to execute
	 * 
	 * @param conn
	 * The database connection to use
	 *
	 * @return int
	 * The number of rows affected by the update.
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static int executeUpdate(Connection conn, String sql) throws SQLException, DBException
    {
		if (Log.METHOD_ENTRY) {
			Log.write("<ENTRY>",true);
		}
        int rowsAffected            = 0;
		Statement stmt 	            = null;

		try {
			// Create a JDBC connection and execute the String
			stmt = conn.createStatement();

			//  If appropriate write out trace information before executing
			if ( Log.SQL_SUMMARY ) {
				Log.write("<SQL_SUMMARY> About to execute SQL: \n" + sql,true);
			}
			rowsAffected = stmt.executeUpdate(sql);

			// Print any warnings chained to statement object
			SQLWarning warning = stmt.getWarnings();
			while( warning != null ){
				Log.write("<SQL Warning :>" + warning.toString(), true);
				warning = warning.getNextWarning() ;
			}

		} 
		catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			Log.write("\n ==========  Database error encountered code  = " + String.valueOf(errCode) + "==========");
            Log.write(sqle);
			Log.write("SQL executed was: " + sql);
			throw new DBException(sqle.getMessage());
		} 
		catch (Exception e) {
			Log.write(e.getMessage());
			throw new DBException(e.getMessage());
		} 
		finally {
			try { stmt.close(); } catch (Exception e) {}
		}

		if (Log.METHOD_EXIT) {
			Log.write("<EXIT>",true);
		}
		return rowsAffected;
    }
    /**
     * Execute the supplied SQL update query against a the supplied
	 * database connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sql
	 * The SQL update to execute
	 * 
	 * @param parms
	 * <CODE>ArrayList</CODE> containing parameters to insert in the sql statement
	 * 
	 * @return int
	 * The number of rows affected by the update.
	 *
	 * @throws DBException
	 * If execution of the update failed.
     */
    public static int executeUpdate(Connection conn, String sql, List parms) throws SQLException, DBException
    {
		if (Log.METHOD_ENTRY) {
			Log.write("<ENTRY>",true);
		}

		int rowsAffected            = 0;
		PreparedStatement ps 	            = null;

		try {
			// Create a JDBC connection and execute the String
			ps = conn.prepareStatement(sql);
			if (parms ==null ) {
				parms = new ArrayList();
			}
			// Set the parameters
			for ( int j=0 ; j<parms.size() ;j++ ) {
				if ( parms.get(j) == null ) {	
					ps.setNull( j+1, Types.NULL );
				} else if ( parms.get(j) instanceof java.lang.Long ) {
					ps.setLong( j+1, ((java.lang.Long)parms.get(j)).longValue() );
				} else if ( parms.get(j) instanceof java.lang.String ) {
					ps.setString( j+1, (java.lang.String)parms.get(j) );
				} else {
					Log.write("Unknown parameter type supplied for UPDATE", true) ; 
					throw new DBException ("Unknown parameter type supplied for UPDATE") ; 
				}
			} 

			//  If appropriate write out trace information before executing
			if ( Log.SQL_SUMMARY ) {
				Log.write("<SQL_SUMMARY> About to execute SQL: \n" + sql,true);
			}
			if ( Log.SQL_VERBOSE ) {			
				Log.write("<SQL_VERBOSE> ... with parameters: ", true) ; 
				for ( int j=0 ; j<parms.size() ;j++ ) {
					Log.write( "<SQL_VERBOSE> ... parm" + String.valueOf(j+1) + ": " + parms.get(j).toString(), true) ; 
				}
			}
			rowsAffected = ps.executeUpdate();

			// Print any warnings chained to statement object
			SQLWarning warning = ps.getWarnings();
			while( warning != null ){
				Log.write("<SQL Warning :>" + warning.toString(), true);
				warning = warning.getNextWarning() ;
			}

		} 
		catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			Log.write("\n ==========  Database error encountered code  = " + String.valueOf(errCode) + "==========");
            Log.write(sqle);
			Log.write("SQL executed was: " + sql);
			throw new DBException(sqle.getMessage());
		} 
		catch (Exception e) {
			Log.write(e.getMessage());
			throw new DBException(e.getMessage());
		} 
		finally {
			try { ps.close(); } catch (Exception e) {}
		}

		if (Log.METHOD_EXIT) {
			Log.write("<EXIT>",true);
		}
		return rowsAffected;
    }
    /**
     * Execute the supplied stored procedure against the supplied 
	 * database connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sp
	 * A string containing the stored procedure name 
	 * 
	 * @param parm[] 
	 * An array of String parameters for the sored procedure
	 *
	 * @return int
	 * The number of rows affected by the stored procedure executed
	 *
	 * @throws DBException
	 * If execution of the stored procedure failed.
     */
    public static int executeDmlProcedure(Connection conn, String sp, ArrayList parms) throws DBException
    {
		if (Log.METHOD_ENTRY) {
			Log.write("<ENTRY>",true);
		}
		List resultList             = null;
		CallableStatement cs        = null;
		int i ; 

        try {
			int j ; 

			StringBuffer spCall = new StringBuffer( "{call " + sp + "(" ) ;

			if (parms ==null ) {
				parms = new ArrayList();
			}

			for ( j=0 ; j<parms.size() ;j++ ) {
				spCall.append("?,") ; 
			}
			// remove the trailing comma
			StringBuffer spCall2 = new StringBuffer( spCall.substring(0, spCall.length()-1) ) ; 
			spCall2.append(")}") ; 
			cs = conn.prepareCall( spCall2.toString() );

			for ( j=0 ; j<parms.size() ;j++ ) {
				if ( parms.get(j) instanceof java.lang.Long ) {
					cs.setLong( j+1, ((java.lang.Long)parms.get(j)).longValue() );
				} else if ( parms.get(j) instanceof java.lang.String ) {
					cs.setString( j+1, (java.lang.String)parms.get(j) );
				} else if ( parms.get(j) instanceof Integer ) {
					cs.setNull( j+1, ((Integer)parms.get(j)).intValue() );
				} else {
					Log.write("Unknown parameter type supplied for stored procedure", true) ; 
					throw new DBException ("Unknown parameter type supplied for stored procedure") ; 
				}
			} 

			// if appropriate write out trace information before executing SQL
			if ( Log.SQL_SUMMARY ) {
				Log.write("<SQL_SUMMARY> About to execute stored procedure: " + sp,true);
			}
			if ( Log.SQL_VERBOSE ) {
				Log.write("<SQL_VERBOSE> ... with parameters: ", true) ; 
				for ( j=0 ; j<parms.size() ;j++ ) {
					if (parms.get(j)!=null) {
						Log.write( "<SQL_VERBOSE> ... " + parms.get(j).getClass().getName() + " parm" + String.valueOf(j+1) + ": " + 
														  parms.get(j).toString(), true) ; 
					} else {
						Log.write( "<SQL_VERBOSE> ... parm" + 
														  String.valueOf(j+1) + ": null", true) ; 
					}
				}
			}
			i=cs.executeUpdate();

			// Print any warnings chained to statement object
            SQLWarning warning = cs.getWarnings();
            while( warning != null ) {
            	Log.write("<SQL Warning>" + warning.toString(), true);
            	warning = warning.getNextWarning() ;
            }

		}
		catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			Log.write("==========  Database error encountered: code  = " + String.valueOf(errCode) + " ==========", true);
            Log.write(sqle);
			Log.write("Stored procedure executed was: " + sp, true);
			throw new DBException(sqle.getMessage());
		}
		catch (Exception e) {
			Log.write(e.getMessage(), true);
			throw new DBException(e.getMessage());
		} 
		finally {
			try { 
				cs.close() ; 
				cs = null ;
			} catch (Exception e) {}
		}

		if (Log.METHOD_EXIT) {
			Log.write("<EXIT>",true);
		}
		return i ;
    }
    /**
     * Execute the supplied stored procedure against the supplied 
	 * database connection.
	 *
	 * @param conn
	 * The database connection to use
	 *
	 * @param sp
	 * A string containing the stored procedure name 
	 * 
	 * @param parm[] 
	 * An array of String parameters for the stored procedure
	 *
	 * @return rs
	 * <CODE>java.sql.ResultSet</CODE> containing any rows found
	 *
	 * @throws DBException
	 * If execution of the stored procedure failed.
     */
    public static java.lang.Object executeQueryProcedure(Connection conn, String sp, List parms, int returnType) throws DBException
    {
		if (Log.METHOD_ENTRY) {
			Log.write("<ENTRY>",true);
		}

		java.lang.Object o		= null ;
		//List resultList         = null;
		CallableStatement cs    = null;
		boolean results			= false ; 
		java.sql.ResultSet rs	= null ; 

        try {
			int j ; 

			StringBuffer spCall = new StringBuffer( "{?=call " + sp + "(" ) ;
			if (parms ==null ) {
				parms = new ArrayList();
			}
			for ( j=0 ; j<parms.size() ;j++ ) {
				spCall.append("?,") ; 
			}
			// remove the trailing comma
			StringBuffer spCall2 = new StringBuffer( spCall.substring(0, spCall.length()-1) ) ; 
			spCall2.append(")}") ; 

			if (Log.SQL_VERBOSE) {
				Log.write( spCall2.toString(), true ) ; 
			}
			cs = conn.prepareCall( spCall2.toString() );

			for ( j=0 ; j<parms.size() ;j++ ) {
				if ( parms.get(j) instanceof java.lang.String ) {
					cs.setString( j+2, (java.lang.String)parms.get(j) );
				} else if ( parms.get(j) instanceof java.lang.Long ) {
					cs.setLong( j+2, ((java.lang.Long)parms.get(j)).longValue() );
				} else {
					Log.write("Unknown parameter type supplied for stored procedure", true) ; 
					throw new DBException ("Unknown parameter type supplied for stored procedure") ; 
				}
			} 

			// if appropriate write out trace information before executing SQL
			if ( Log.SQL_SUMMARY ) {
				Log.write("<SQL_SUMMARY> About to execute stored procedure: " + sp, true);
			}
			if ( Log.SQL_VERBOSE ) {

				Log.write("<SQL_VERBOSE> ... with parameters: ", true) ; 
				for ( j=0 ; j<parms.size() ;j++ ) {
					Log.write( "<SQL_VERBOSE> ... parm" + String.valueOf(j+1) + ": " + parms.get(j).toString(), true) ; 
				}
			}

			rs=cs.executeQuery();
			
			if ( rs == null ) {
				Log.write("<SQL_VERBOSE> failed to get result set object ", true); 
			}
			// Do any object conversions required 
			switch (returnType) {
				case 1:
					List resultList = convertResultToList(rs) ; 					
					o = resultList; 
					break;
                case 2:
/* Sybase proprietary functionality 
                    TabularResults.ResultSet resultTRS = null ; 
					try {
						resultTRS = IDL.getResultSet(rs) ; 					
					}
					catch (SQLException e) {
						Log.write("Conversion to Tabular Result Set fails", true); 
					}
					o = resultTRS ; 
*/
                    throw new DBException("Attempt to use Sybase proprietary technology that has been compiled out of this version") ; 
				default :
					o = rs ; 
			}

			// Print any warnings chained to statement object
            SQLWarning warning = cs.getWarnings();
            while( warning != null ) {
            	Log.write("<SQL Warning>" + warning.toString());
            	warning = warning.getNextWarning() ;
            }

		}
		catch (SQLException sqle) {
			int errCode = sqle.getErrorCode();
			Log.write("==========  Database error encountered: code  = " + String.valueOf(errCode) + " ==========", true);
            Log.write(sqle);
			Log.write("Stored procedure executed was: " + sp, true);
			throw new DBException(sqle.getMessage());
		}
		catch (Exception e) {
			Log.write(e.getMessage());
			throw new DBException(e.getMessage());
		} 
		finally {
			try { cs.close(); } catch (Exception e) {}
		}

		if (Log.METHOD_EXIT) {
			Log.write("<EXIT>",true);
		}
		return o ;
	} 
    /**
     * convertResultToList Method
	 * 
	 * @return list		
	 * An <CODE>ArrayList</CODE> of <CODE>TreeMap</CODE>s. Each
	 * map contains the column heading value pairs for a single result set
	 * row.
     */
    public static List convertResultToList(java.sql.ResultSet sqlResults) throws DBException
    {
		if (Log.METHOD_ENTRY) {
			Log.write("<ENTRY>",true);
		}
		Map map   = null;
		List list = new ArrayList();
		StringBuffer sbuff=new StringBuffer();

		try {
			// get meta data
			ResultSetMetaData meta=sqlResults.getMetaData();
			int colCount=meta.getColumnCount();
			int rowCount=0;

			// get column headings
			String[] headings = new String[colCount];
        	for(int i = 0 ; i < colCount; i++) {
        		headings[i] = meta.getColumnLabel(i + 1); 
				if (headings[i] == null) {
					headings[i] = "column " + String.valueOf(i);
				}
        	}

			//Process all results in set
			while ( sqlResults.next() == true ) {

				map=new HashMap(colCount);

				// Processes a single row of the result set.
				// Each column becomes an element in the Map keyed
				// by the column heading
				for( int i = 0; i < colCount; i++) {

					// Get column value 
            		// Remember column numbers start from 1 !!!!
					Object resObj = sqlResults.getObject(i + 1);

					// Print out the retrieved info for verbose logging
					if ( Log.SQL_VERBOSE ) {
						Log.write("Column "  + String.valueOf(i + 1 ) + ": " + headings[i] + " : " 
							+ ((resObj== null) ? "null" : resObj.toString()), true);
					}

					// Append value to string for SQL logging
					if ( Log.SQL_VERBOSE ) {
						sbuff.append(String.valueOf(resObj));
						sbuff.append("|");
					}

					// Store in map
					if(! sqlResults.wasNull()) {
                		if (resObj instanceof String) {
							map.put(headings[i], ((String)resObj).trim());
                   		} else {
                        	map.put(headings[i], resObj);
                    	}
                	}

					// send result trace to log
					if ( Log.SQL_VERBOSE ) {
						Log.write("<SQL RESULT>" + sbuff.toString(), true);
                    	sbuff.setLength(0);
					}
				}

				TreeMap treeMap=new TreeMap(map);
				list.add(treeMap);
			}
			
			// send result information to the log
			if ( Log.SQL_RESULT ) {
				Log.write("<SQL_RESULT> Got results set with " + String.valueOf(list.size()) + " rows and " + 
														  String.valueOf(colCount) + " columns", true);
			}

		} catch (Exception e) {
			Log.write(e.getMessage());
			throw new DBException(e.getMessage());
		}

		if (Log.METHOD_EXIT) {
			Log.write("<EXIT>",true);
		}
		return list;
    }
	
	/** Valid Object type for result set returns */
	public static final int LIST = 1; 
	/** Valid Object type for result set returns */
	public static final int TABULAR_RESULT_SET = 2; 
	/** Valid Object type for result set returns */
	public static final int SQL_RESULT_SET = 3; 
}

