package org.thestephensons.db;

/*
 * PROJECT			: database integration library
 *  
 * COPYRIGHT        : Copyright (C) 2001 Tim Stephenson
 *
 * Change History:
  * -------------------------------------------------------------------
 * Version	Date	Author				Description
 * 1.0		28Apr01 Tim Stephenson		Created.
 * $Log: SqlType.java,v $
 * Revision 1.1  2001/06/01 08:37:19  tstephen
 * lots of untracked changes leading to v1.0
 *
 */

import java.io.Serializable;
import java.sql.Types;

/**
 * <p>This class has a primary purpose of representing a null value to be 
 * inserted into some SQL call handled by the proxy objects</p>
 * 
 * <p>This is necessary to allow the proxies to receive a null object 
 * yet still provide the JDBC driver with a known object type 
 * as required by <CODE>PreparedStatement.setNull</CODE></p>
 * 
 * <p>The integer used in the constructor should be one of the static 
 * final members of <CODE>Types</CODE></p>
 * 
 * @version __VERSION__
 * 
 * @author tim@thestephensons.org
 * 
 * @see java.sql.Types java.sql.Types                       <br/>
 * @see org.thestephensons.db.DatabaseProxy DatabaseProxy   <br/>
 * @see org.thestephensons.db.GenericDBProxy GenericDBProxy <br/>
 */
public class SqlType implements Serializable {

	/**
	 * Represents a Boolean
	 */
	public static final int BOOLEAN = Types.BIT;
	/**
	 * Represents a Character
	 */
	public static final int CHARACTER = Types.CHAR;
	/**
	 * Represents a Double
	 */
	public static final int DOUBLE = Types.DOUBLE;
	/**
	 * Represents a Float
	 */
	public static final int FLOAT = Types.FLOAT;
	/**
	 * Represents an Integer
	 * @deprecated
	 */
	public static final int INTEGER = Types.INTEGER;
	/**
	 * Represents a Long
	 */
	public static final int LONG = Types.NUMERIC;
	/**
	 * Represents a Short
	 */
	public static final int SHORT = Types.SMALLINT;
	/**
	 * Represents a String
	 */
	public static final int STRING = Types.VARCHAR;
	/**
	 * Represents a BigDecimal
	 */
	public static final int BIGDECIMAL = Types.NUMERIC;
	/**
	 * Represents a java.util.Date
	 */
	public static final int DATE = Types.DATE;
	/**
	 * Represents a Byte[]
	 */
	public static final int BYTEARRAY = Types.BLOB;
	/**
	 * Represents a SQL Date
	 */
	public static final int SQLDATE = Types.DATE;
	/**
	 * Represents a SQL Time
	 */
	public static final int SQLTIME = Types.TIME;
	/*
	 * Represents a SQL Timestamp.
	 * @deprecated
	 */
	public static final int SQLTIMESTAMP = Types.TIMESTAMP;

	/** 
	 * Returns a string name of the datatype represented by this object
	 */
	public String toString() {
        String dataTypeName ; 

		switch (dataType) {
		case Types.CHAR:
		    dataTypeName = "char" ; 
            dataTypeName += "(" ;
            dataTypeName += String.valueOf(scale) ;
            dataTypeName += ")" ; 
			break ; 
		case Types.DATE:
		    dataTypeName = "date" ; 
			break ; 
		case Types.INTEGER:
		    dataTypeName = "int" ; 
			break ; 
		case Types.NUMERIC:
		    dataTypeName = "numeric" ; 
            dataTypeName += "(" ;
            dataTypeName += String.valueOf(scale) ;
            dataTypeName += "," ;
            dataTypeName += String.valueOf(precision) ;
            dataTypeName += ")" ; 
			break ; 
		case Types.SMALLINT:
		    dataTypeName = "smallint" ; 
			break ; 
		case Types.TINYINT:
		    dataTypeName = "tinyint" ; 
			break ; 
		case Types.TIMESTAMP:
		    dataTypeName = "datetime" ; 
			break ; 
		case Types.VARCHAR:
		    dataTypeName = "varchar" ; 
            dataTypeName += "(" ;
            dataTypeName += String.valueOf(scale) ;
            dataTypeName += ")" ; 
            break ; 
		default : 
		    dataTypeName = "NotImplemented" ; 
		}

        return dataTypeName ; 
	}

	/**
	 * Constructs a new instance of this class initialised with the integer constant of its type.
	 * @param dataType the int that represents the datatype that this class is being created for.
	 */
	public SqlType(int dataType, int scale, int precision) {
		this.dataType = dataType;
		this.scale = scale;
		this.precision = precision;
	}
	
	/**
	 * Returns the datatype that this class represents
	 * @returns int the datatype that this class is representative of.
	 */
	public int getDataType() {
		return dataType;
	}

	/** the internal storage for this dataType. */
	private int dataType = 0;
	/** the internal storage for this data type's scale */
	private int scale = 0;
	/** the internal storage for this data type's precision */
	private int precision = 0;

    /** 
     * Information on the exact CVS version accessible after compilation
     */
    public static final String about = "$Revision: 1.1 $";

}

